@@ -210,6 +210,8 @@ _XcmsParseColorString(
      */
     if ((len = strlen(color_string)) >= sizeof(string_buf)) {
 	string_lowered = Xmalloc(len+1);
+	if (string_lowered == NULL)
+	    return(XcmsFailure);
     } else {
 	string_lowered = string_buf;
     }
@@ -419,6 +421,8 @@ _XcmsLookupColorName(
 Retry:
     if ((len = strlen(tmpName)) > 63) {
 	name_lowered = Xmalloc(len+1);
+	if (name_lowered == NULL)
+	    return(XcmsFailure);
     } else {
 	name_lowered = name_lowered_64;
     }
@@ -762,6 +766,13 @@ LoadColornameDB(void)
 
     strings = Xmalloc(size);
     pairs = Xcalloc(nEntries, sizeof(XcmsPair));
+    if (strings == NULL || pairs == NULL) {
+	free(strings);
+	free(pairs);
+	(void) fclose(stream);
+	XcmsColorDbState = XcmsDbInitFailure;
+	return(XcmsFailure);
+    }
 
     ReadColornameDB(stream, pairs, strings);
     (void) fclose(stream);
